from typing import Annotated
from contextlib import asynccontextmanager
from pathlib import Path
from fastapi import FastAPI, Query, HTTPException
from fastapi.responses import HTMLResponse, RedirectResponse
from sqlmodel import select


from agentbeats.model import (
    create_db_and_tables,
    SessionDep,
    Agent,
    AgentDeployType,
    AgentHostedStatus,
)


@asynccontextmanager
async def lifespan(app: FastAPI):
    create_db_and_tables()
    yield


app = FastAPI(lifespan=lifespan)


@app.post("/agents/")
def create_agent(agent: Agent, session: SessionDep) -> Agent:
    if agent.id is not None:
        raise HTTPException(status_code=400, detail="ID should not be provided")
    if agent.created_at is not None:
        raise HTTPException(status_code=400, detail="created_at should not be provided")
    if agent.hosted_status != AgentHostedStatus.PENDING:
        raise HTTPException(
            status_code=400, detail="hosted_status should not be provided"
        )
    if agent.deploy_type == AgentDeployType.HOSTED:
        git_fields_provided = (agent.git_url is not None) and (
            agent.git_branch is not None
        )
        docker_field_provided = agent.docker_image_url is not None
        if not (git_fields_provided or docker_field_provided):
            raise HTTPException(
                status_code=400,
                detail="For hosted agents, either git_url and git_branch or docker_image_url must be provided",
            )
    else:
        if agent.git_url is not None or agent.git_branch is not None:
            raise HTTPException(
                status_code=400,
                detail="git_url and git_branch should be null for non-hosted agents",
            )
        if agent.docker_image_url is not None:
            raise HTTPException(
                status_code=400,
                detail="docker_image_url should be null for non-hosted agents",
            )
    session.add(agent)
    session.commit()
    session.refresh(agent)
    return agent


@app.get("/agents/")
def read_agents(
    session: SessionDep,
    offset: int = 0,
    limit: Annotated[int, Query(ge=1, le=100)] = 100,
) -> list[Agent]:
    agents = session.exec(select(Agent).offset(offset).limit(limit)).all()
    return list(agents)


@app.get("/agents/{agent_id}")
def read_agent(agent_id: int, session: SessionDep) -> Agent:
    agent = session.get(Agent, agent_id)
    if not agent:
        raise HTTPException(status_code=404, detail="Agent not found")
    return agent


@app.delete("/agents/{agent_id}")
def delete_agent(agent_id: int, session: SessionDep) -> dict:
    agent = session.get(Agent, agent_id)
    if not agent:
        raise HTTPException(status_code=404, detail="Agent not found")
    session.delete(agent)
    session.commit()
    return {"message": "Agent deleted"}


@app.get("/test_agent_crud", response_class=HTMLResponse)
async def test_agent_crud():
    """Serve the Agent CRUD frontend interface"""
    html_path = Path(__file__).parent.parent / "frontend" / "agent_crud.html"
    return html_path.read_text()


@app.get("/")
async def root():
    # redirect to test_agent_crud
    return RedirectResponse(url="/test_agent_crud")
    # return {"message": "Hello World"}



from enum import Enum
from typing import Annotated
from datetime import datetime
from fastapi import Depends
from sqlmodel import SQLModel, Session, Field, create_engine
from sqlalchemy import Column, DateTime, func


class AgentDeployType(str, Enum):
    HOSTED = "hosted"
    REMOTE = "remote"
    PROXIED = "proxied"


class AgentHostedStatus(str, Enum):
    PENDING = "pending"  # or not hosted
    STARTING = "starting"
    RUNNING = "running"
    ERROR = "error"


class Agent(SQLModel, table=True):
    id: int | None = Field(default=None, primary_key=True)
    name: str = Field(index=True)
    created_at: datetime = Field(
        sa_column=Column(
            DateTime(timezone=True), server_default=func.now(), index=True
        ),
    )
    # agent types
    is_green: bool = Field(index=True)
    deploy_type: AgentDeployType = Field(index=True)
    # agent config
    secret: str
    allow_key_creation: bool
    git_url: str | None
    git_branch: str | None
    docker_image_url: str | None
    ctrl_url: str | None
    # status
    hosted_status: AgentHostedStatus = Field(
        default=AgentHostedStatus.PENDING, index=True
    )


sqlite_file_name = "database.db"
sqlite_url = f"sqlite:///{sqlite_file_name}"


connect_args = {"check_same_thread": False}
engine = create_engine(sqlite_url, connect_args=connect_args)


def create_db_and_tables():
    SQLModel.metadata.create_all(engine)


def get_session():
    with Session(engine) as session:
        yield session


SessionDep = Annotated[Session, Depends(get_session)]



import os
import typer
import uvicorn
from agentbeats.backend import app as backend_app
from agentbeats.controller import main as controller_main

typer_app = typer.Typer()


@typer_app.command()
def serve():
    host = os.getenv("HOST", "127.0.0.1")
    port = int(os.getenv("PORT", "8000"))
    uvicorn.run(backend_app, host=host, port=port)


typer_app.command("run_ctrl")(controller_main)


if __name__ == "__main__":
    typer_app()
